#!/usr/bin/env python
# -*- coding: utf-8 -*-

VERSION = (0, 3, 1)

__author__ = 'Tzu-ping Chung'
__email__ = 'uranusjr@gmail.com'
__version__ = '.'.join(str(v) for v in VERSION)

# If we import directly, this may fail if six is not installed first.
# TODO: Find a better workaround for this problem.
try:
    from .plugin import SphinxDoctest   # noqa
except ImportError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import

import errno
import os

from sphinx.application import Sphinx
from sphinx.ext.doctest import DocTestBuilder


class SphinxDocTestCollectorBuilder(DocTestBuilder):
    """A fake Sphinx builder that collects tests from docs.
    """
    def __init__(self, app, into, excludes):
        super(SphinxDocTestCollectorBuilder, self).__init__(app)
        self.into = into
        self.excludes = excludes

    def test_group(self, group, filename):
        if filename not in self.excludes:
            self.into.append((filename, group,))


class DocTestCollector(Sphinx):

    def __init__(self, groups, excludes, **options):
        options.update({
            'buildername': 'doctest',   # Pretend that we're running doctests.
            'status': None,             # Silence Sphinx output.
            'warning': None,
        })
        super(DocTestCollector, self).__init__(**options)

        # Swap-out Sphinx's doctest builder with our own.
        self.builder = SphinxDocTestCollectorBuilder(
            app=self, into=groups, excludes=excludes,
        )


def make_build_dirs(builddir):
    outdir = os.path.join(builddir, 'doctest')
    doctreedir = os.path.join(builddir, 'doctrees')

    build_dirs = (outdir, doctreedir,)
    for build_dir in build_dirs:
        try:
            os.mkdir(build_dir)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise
    return build_dirs


def collect_sphinx_doctests(srcdir, confdir, builddir, excludes):
    """Collect doctests from a Sphinx documentation setup.
    """
    outdir, doctreedir = make_build_dirs(builddir)

    groups = []

    # Run Sphinx to collect doctest groups.
    sphinx = DocTestCollector(
        groups=groups, srcdir=srcdir, confdir=confdir, outdir=outdir,
        doctreedir=doctreedir, excludes=excludes,
    )
    sphinx.builder.build_update()
    sphinx.builder.cleanup()

    return groups



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import doctest
import glob
import os

import six
from nose.plugins import Plugin

from .finders import SphinxDocTestFinder
from .sphinx import collect_sphinx_doctests


def get_files_from_patterns(patterns, working_dir):
    cwd = os.getcwd()
    os.chdir(working_dir)
    excludes = set()
    for pat in patterns:
        excludes.update(glob.iglob(pat))
    os.chdir(cwd)
    return excludes


class CodeExecutor(object):
    """Global code executor to patch doctest's compile function dynamically.

    Based on the monkey-patching approach in Sphinx's doctest plugin.
    """
    compile_type = 'single'

    def compile(self, code, name, type, flags, dont_inherit):
        return compile(code, name, self.compile_type, flags, dont_inherit)


executor = CodeExecutor()


class DocTestCase(doctest.DocTestCase):
    """Custom DocTestCast that does not clear globals after tests are run.

    Built-in doctest's implemetation clears test's globals on tear down. We
    override the method to prevent that.
    """
    def tearDown(self):
        if self._dt_tearDown is not None:
            self._dt_tearDown(self._dt_test)


class SphinxDoctest(Plugin):
    """The plugin object to be registered into Nose.
    """
    name = 'sphinx'
    enabled = True

    def options(self, parser, env):
        """Setup command-line options to parse.

        Overriden from Plugin.
        """
        super(SphinxDoctest, self).options(parser, env)
        parser.add_option(
            '--sphinx-doc-dir', dest='sphinx_doc_dir',
            default=env.get('NOSE_SPHINX_DOC_DIR', 'docs'),
            help=('Root directory of the Sphinx documentation. Default is '
                  '"docs". [NOSE_SPHINX_DOC_DIR]'),
        )
        parser.add_option(
            '--sphinx-conf-dir', dest='sphinx_conf_dir',
            default=env.get('NOSE_SPHINX_CONF_DIR', None),
            help=('Directory of the Sphinx documentation containing '
                  '"conf.py". Defaults to the Sphinx root directory. '
                  '[NOSE_SPHINX_CONF_DIR]'),
        )
        parser.add_option(
            '--sphinx-build-dir', dest='sphinx_build_dir',
            default=env.get('NOSE_SPHINX_BUILD_DIR', '_build'),
            help=('Path to the Sphinx build directory, relative to the root '
                  'documentation directory. Default is "_build". '
                  '[NOSE_SPHINX_BUILD_DIR]'),
        )
        parser.add_option(
            '--sphinx-exclude', action='append', dest='sphinx_excludes',
            default=env.get('NOSE_SPHINX_EXCLUDE', []),
            help=('Doctests inside files specified here will not be '
                  'collectied. You can use glob patterns. May be specified '
                  'multiple times. [NOSE_SPHINX_EXCLUDE]'),
        )

    def configure(self, options, config):
        """Configure the plugin from parsed command-line options.

        Overriden from Plugin.
        """
        self.srcdir = options.sphinx_doc_dir
        self.confdir = options.sphinx_conf_dir
        if self.confdir is None:
            self.confdir = self.srcdir
        self.builddir = os.path.join(self.srcdir, options.sphinx_build_dir)
        self.excludes = get_files_from_patterns(
            options.sphinx_excludes, self.srcdir,
        )

    def begin(self):
        """Run before start of "the" test.

        Overriden from Plugin.
        """
        # Monkey-patch doctest's compile to provide more functionality.
        # Based on Sphinx's implementation.
        try:
            self.old_compile = doctest.compile
        except AttributeError:
            pass
        doctest.compile = executor.compile

        # Monkey-patch doctest's DocTestCase to prevent globals being cleared.
        # See DocTestCase's implementation.
        self.DocTestCase = doctest.DocTestCase
        doctest.DocTestCase = DocTestCase

    def finalize(self, result):
        """Run after end of "the" test.

        Overriden from Plugin.
        """
        # See begin().
        try:
            doctest.compile = self.old_compile
        except AttributeError:
            del doctest.compile
        doctest.DocTestCase = self.DocTestCase

    def wantDirectory(self, dirname):
        """Specify directories we want.

        Overriden from Plugin. We want directories that contain a valid Sphinx
        doc tree. No preferences about others.
        """
        if self._path_has_sphinx(dirname):
            return True
        return None

    def loadTestsFromDir(self, path):
        """Load tests from Sphinx subdirectory in the given directory.

        Overriden from Plugin.
        """
        # We only run for directories we really want.
        if not self.wantDirectory(path):
            return

        kwargs = self._get_sphinx_dirs(path)
        kwargs['excludes'] = self.excludes
        groups = collect_sphinx_doctests(**kwargs)

        for docname, group in groups:

            # All tests in a group share their global context.
            group.globs = {}

            def build_setup_teardown(fname, group, before=False, after=False):

                def func(test):
                    # All tests in a group share their global context.
                    test.globs = group.globs

                    # Set type for the monkey-patched compile function.
                    executor.compile_type = test.compile_type

                    # Run setup/cleanup in the current context.
                    for setup_code in group.setup:
                        code_name = '<{kind} {docname}:{name}>'.format(
                            kind=fname, docname=docname, name=group.name,
                        )
                        six.exec_(
                            compile(setup_code.code, code_name, 'exec', 0, 1),
                            test.globs,
                        )

                func.__name__ = fname
                return func

            try:
                suite = doctest.DocTestSuite(
                    test_finder=SphinxDocTestFinder(docname, group),
                    setUp=build_setup_teardown('setup', group),
                    tearDown=build_setup_teardown('teardown', group),
                )
            except ValueError:  # No docstring. Ignore.
                continue
            yield suite

    def _get_sphinx_dirs(self, path):
        """Resolve Sphinx directories with the given path.
        """
        return {
            key: os.path.join(os.path.abspath(path), getattr(self, key))
            for key in ('srcdir', 'confdir', 'builddir',)
        }

    def _path_has_sphinx(self, path):
        """Check for Sphinx directories in path.
        """
        sphinx_dirs = self._get_sphinx_dirs(path)
        return all(os.path.exists(p) for p in sphinx_dirs.values())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sphinxnose
from setuptools import find_packages, setup

readme = open('README.rst').read()

requirements = open('requirements.txt').read().strip().split('\n')

setup(
    name='sphinx-nose',
    version=sphinxnose.__version__,
    description='Run Sphinx doctests in Nose.',
    long_description=readme,
    author='Tzu-ping Chung',
    author_email='uranusjr@gmail.com',
    url='https://github.com/uranusjr/saim-al-dahr',
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    license='MIT',
    zip_safe=False,
    keywords=['doctest', 'nose', 'sphinx'],
    entry_points={
        'nose.plugins.0.10': [
            'sphinx = sphinxnose:SphinxDoctest',
        ],
    },
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import doctest

parser = doctest.DocTestParser()


class SphinxDocTestFinder(object):
    """Doctest finder for doctest.DocTestSuite.
    """
    def __init__(self, filename, group):
        super(SphinxDocTestFinder, self).__init__()
        self.filename = filename
        self.group = group

    def find(self, obj, name=None, module=None, globs=None, extraglobs=None):
        """Hook called by the test runner to find tests.

        In reality (within context of Nose and Sphinx), almost all parameters
        provided are not used. Tests are constructed from Sphinx doctest groups
        provided in __init__.
        """
        # Tests are going to be collected here.
        tests = []

        # Based on Sphinx's doctest collecting logic, but instead of running
        # the tests directly, we create DocTest objects and return them for
        # nose to collect and run.
        # Comments here belong to Sphinx if they are "written like this".
        # My own comments "Capitalize the first letter and have a punctuation
        # mark at the end."
        for code in self.group.tests:
            if len(code) == 1:
                # ordinary doctests (code/output interleaved)
                try:
                    test = parser.get_doctest(
                        code[0].code, {}, self.group.name, self.filename,
                        code[0].lineno,
                    )
                except Exception:
                    # TODO: Warn about invalid doctest?
                    continue
                if not test.examples:
                    continue
                for example in test.examples:
                    # apply directive's comparison options
                    new_opt = code[0].options.copy()
                    new_opt.update(example.options)
                    example.options = new_opt
                test.compile_type = 'single'
            else:
                # testcode and output separate
                output = code[1] and code[1].code or ''
                options = code[1] and code[1].options or {}
                # disable <BLANKLINE> processing as it is not needed
                options[doctest.DONT_ACCEPT_BLANKLINE] = True
                # find out if we're testing an exception
                m = parser._EXCEPTION_RE.match(output)
                if m:
                    exc_msg = m.group('msg')
                else:
                    exc_msg = None
                example = doctest.Example(
                    code[0].code, output, exc_msg=exc_msg,
                    lineno=code[0].lineno, options=options,
                )
                # The test's globals will be swapped out later during test
                # setup, so the globs parameter here has no use at all, and an
                # empty dict is enough.
                test = doctest.DocTest(
                    [example], {}, self.group.name, self.filename,
                    code[0].lineno, None,
                )
                test.compile_type = 'exec'
            test.group = self.group
            tests.append(test)

        # Sort tests by names alphabetically, for consistency with
        # doctest.DocTestFinder's behavior.
        tests.sort()
        return tests
